"""
Constants that will be required for the models
"""

cG = 6.67408e-8  # Gravitational constant in CGS
ckB = 1.38064852e-16  # Boltzmann
cmH = 1.6735328363560003e-24  # Mass of a Hydrogen atom in grams
cparsec = 3.08567758123e18  # 1 parsec in cm
csolarmass = 1.989e33  # Solar mass in grams
cyear = 60 * 60 * 24 * 365  # 1 year in seconds
